==> future.BatchJobs_0.12.1.tar.gz: future.BatchJobs/tests/batchjobs_multicore.R <==
source("incl/start.R")
library("listenv")


message("*** batchjobs_multicore() ...")

for (cores in 1:min(3L, availableCores("multicore"))) {
  ## FIXME: 
  if (!fullTest && cores > 1) next
  
  message(sprintf("Testing with %d cores ...", cores))
  options(mc.cores=cores-1L)

  if (!supportsMulticore()) {
    message(sprintf("BatchJobs multicore futures are not supporting on '%s'. Falling back to use synchroneous BatchJobs local futures", .Platform$OS))
  }

  for (globals in c(FALSE, TRUE)) {

  message(sprintf("*** batchjobs_multicore(..., globals=%s) without globals", globals))

  f <- batchjobs_multicore({
    42L
  }, globals=globals)
  stopifnot(inherits(f, "BatchJobsFuture") || ((cores == 1 || !supportsMulticore()) && inherits(f, "EagerFuture")))

  print(resolved(f))
  y <- value(f)
  print(y)
  stopifnot(y == 42L)


  message(sprintf("*** batchjobs_multicore(..., globals=%s) with globals", globals))
  ## A global variable
  a <- 0
  f <- batchjobs_multicore({
    b <- 3
    c <- 2
    a * b * c
  }, globals=globals)
  print(f)


  ## A multicore future is evaluated in a separated
  ## forked process.  Changing the value of a global
  ## variable should not affect the result of the
  ## future.
  a <- 7  ## Make sure globals are frozen
##  if ("covr" %in% loadedNamespaces()) v <- 0 else ## WORKAROUND
  v <- value(f)
  print(v)
  stopifnot(v == 0)


  message(sprintf("*** batchjobs_multicore(..., globals=%s) with globals and blocking", globals))
  x <- listenv()
  for (ii in 1:4) {
    message(sprintf(" - Creating batchjobs_multicore future #%d ...", ii))
    x[[ii]] <- batchjobs_multicore({ ii }, globals=globals)
  }
  message(sprintf(" - Resolving %d batchjobs_multicore futures", length(x)))
##  if ("covr" %in% loadedNamespaces()) v <- 1:4 else ## WORKAROUND
  v <- sapply(x, FUN=value)
  stopifnot(all(v == 1:4))


  message(sprintf("*** batchjobs_multicore(..., globals=%s) and errors", globals))
  f <- batchjobs_multicore({
    stop("Whoops!")
    1
  }, globals=globals)
  print(f)
  v <- value(f, signal=FALSE)
  print(v)
  stopifnot(inherits(v, "simpleError"))

  res <- try(value(f), silent=TRUE)
  print(res)
  stopifnot(inherits(res, "try-error"))

  ## Error is repeated
  res <- try(value(f), silent=TRUE)
  print(res)
  stopifnot(inherits(res, "try-error"))

  } # for (globals ...)


  message("*** batchjobs_multicore(..., workers=1L) ...")

  a <- 2
  b <- 3
  yTruth <- a * b

  f <- batchjobs_multicore({ a * b }, workers=1L)
  rm(list=c("a", "b"))

  v <- value(f)
  print(v)
  stopifnot(v == yTruth)

  message("*** batchjobs_multicore(..., workers=1L) ... DONE")

  message(sprintf("Testing with %d cores ... DONE", cores))
} ## for (cores ...)

message("*** batchjobs_multicore() ... DONE")

source("incl/end.R")

==> future.BatchJobs_0.14.0.tar.gz: future.BatchJobs/tests/batchjobs_multicore.R <==
source("incl/start.R")
library("listenv")


message("*** batchjobs_multicore() ...")

for (cores in 1:min(3L, availableCores("multicore"))) {
  ## FIXME: 
  if (!fullTest && cores > 1) next
  
  message(sprintf("Testing with %d cores ...", cores))
  options(mc.cores=cores-1L)

  if (!supportsMulticore()) {
    message(sprintf("BatchJobs multicore futures are not supporting on '%s'. Falling back to use synchroneous BatchJobs local futures", .Platform$OS.type))
  }

  for (globals in c(FALSE, TRUE)) {

  message(sprintf("*** batchjobs_multicore(..., globals=%s) without globals", globals))

  f <- batchjobs_multicore({
    42L
  }, globals=globals)
  stopifnot(inherits(f, "BatchJobsFuture") || ((cores == 1 || !supportsMulticore()) && inherits(f, "EagerFuture")))

  print(resolved(f))
  y <- value(f)
  print(y)
  stopifnot(y == 42L)


  message(sprintf("*** batchjobs_multicore(..., globals=%s) with globals", globals))
  ## A global variable
  a <- 0
  f <- batchjobs_multicore({
    b <- 3
    c <- 2
    a * b * c
  }, globals=globals)
  print(f)


  ## A multicore future is evaluated in a separated
  ## forked process.  Changing the value of a global
  ## variable should not affect the result of the
  ## future.
  a <- 7  ## Make sure globals are frozen
##  if ("covr" %in% loadedNamespaces()) v <- 0 else ## WORKAROUND
  if (globals) {
    v <- value(f)
    print(v)
    stopifnot(v == 0)
  } else {
    res <- tryCatch({ value(f) }, error=identity)
    print(res)
    stopifnot(inherits(res, "simpleError"))
  }


  message(sprintf("*** batchjobs_multicore(..., globals=%s) with globals and blocking", globals))
  x <- listenv()
  for (ii in 1:4) {
    message(sprintf(" - Creating batchjobs_multicore future #%d ...", ii))
    x[[ii]] <- batchjobs_multicore({ ii }, globals=globals)
  }
  message(sprintf(" - Resolving %d batchjobs_multicore futures", length(x)))
##  if ("covr" %in% loadedNamespaces()) v <- 1:4 else ## WORKAROUND
  if (globals) {
    v <- sapply(x, FUN=value)
    stopifnot(all(v == 1:4))
  } else {
    v <- lapply(x, FUN=function(f) tryCatch(value(f), error=identity))
    stopifnot(all(sapply(v, FUN=inherits, "simpleError")))
  }

  message(sprintf("*** batchjobs_multicore(..., globals=%s) and errors", globals))
  f <- batchjobs_multicore({
    stop("Whoops!")
    1
  }, globals=globals)
  print(f)
  v <- value(f, signal=FALSE)
  print(v)
  stopifnot(inherits(v, "simpleError"))

  res <- try(value(f), silent=TRUE)
  print(res)
  stopifnot(inherits(res, "try-error"))

  ## Error is repeated
  res <- try(value(f), silent=TRUE)
  print(res)
  stopifnot(inherits(res, "try-error"))

  } # for (globals ...)


  message("*** batchjobs_multicore(..., workers=1L) ...")

  a <- 2
  b <- 3
  yTruth <- a * b

  f <- batchjobs_multicore({ a * b }, workers=1L)
  rm(list=c("a", "b"))

  v <- value(f)
  print(v)
  stopifnot(v == yTruth)

  message("*** batchjobs_multicore(..., workers=1L) ... DONE")

  message(sprintf("Testing with %d cores ... DONE", cores))
} ## for (cores ...)

message("*** batchjobs_multicore() ... DONE")

source("incl/end.R")
